@@ -3,28 +3,28 @@
  * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
-#include <signal.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <string.h>
-#include <unistd.h>
-#include <limits.h>
+#include "debug.h"
+#include "hostport.h"
+#include "radsecproxy.h"
+#include "util.h"
+#include <arpa/inet.h>
+#include <ctype.h>
 #include <fcntl.h>
+#include <limits.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#include <openssl/err.h>
+#include <openssl/ssl.h>
+#include <poll.h>
+#include <pthread.h>
+#include <regex.h>
+#include <signal.h>
+#include <string.h>
+#include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <poll.h>
-#include <ctype.h>
 #include <sys/wait.h>
-#include <arpa/inet.h>
-#include <regex.h>
-#include <pthread.h>
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "radsecproxy.h"
-#include "hostport.h"
-#include "debug.h"
-#include "util.h"
+#include <unistd.h>
 
 #ifdef RADPROT_TLS
 
@@ -38,24 +38,24 @@ void tlssetsrcres(void);
 
 static const struct protodefs protodefs = {
     "tls",
-    "radsec", /* secretdefault */
-    SOCK_STREAM, /* socktype */
-    "2083", /* portdefault */
-    0, /* retrycountdefault */
-    0, /* retrycountmax */
-    REQUEST_RETRY_INTERVAL * REQUEST_RETRY_COUNT, /* retryintervaldefault */
-    60, /* retryintervalmax */
-    DUPLICATE_INTERVAL, /* duplicateintervaldefault */
-    setprotoopts, /* setprotoopts */
-    getlistenerargs, /* getlistenerargs */
-    tlslistener, /* listener */
-    tlsconnect, /* connecter */
-    tlsclientrd, /* clientconnreader */
-    clientradputtls, /* clientradput */
-    NULL, /* addclient */
-    NULL, /* addserverextra */
-    tlssetsrcres, /* setsrcres */
-    NULL /* initextra */
+    "radsec",                                    /* secretdefault */
+    SOCK_STREAM,                                 /* socktype */
+    "2083",                                      /* portdefault */
+    0,                                           /* retrycountdefault */
+    0,                                           /* retrycountmax */
+    REQUEST_RETRY_INTERVAL *REQUEST_RETRY_COUNT, /* retryintervaldefault */
+    60,                                          /* retryintervalmax */
+    DUPLICATE_INTERVAL,                          /* duplicateintervaldefault */
+    setprotoopts,                                /* setprotoopts */
+    getlistenerargs,                             /* getlistenerargs */
+    tlslistener,                                 /* listener */
+    tlsconnect,                                  /* connecter */
+    tlsclientrd,                                 /* clientconnreader */
+    clientradputtls,                             /* clientradput */
+    NULL,                                        /* addclient */
+    NULL,                                        /* addserverextra */
+    tlssetsrcres,                                /* setsrcres */
+    NULL                                         /* initextra */
 };
 
 static struct addrinfo *srcres = NULL;
@@ -77,7 +77,7 @@ static char **getlistenerargs(void) {
 
 void tlssetsrcres(void) {
     if (!srcres)
-	srcres =
+        srcres =
             resolvepassiveaddrinfo(protoopts ? protoopts->sourcearg : NULL,
                                    AF_UNSPEC, NULL, protodefs.socktype);
 }
@@ -112,9 +112,9 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
         server->state = RSP_SERVER_STATE_RECONNECTING;
     pthread_mutex_unlock(&server->lock);
 
-    if(server->conf->source) {
+    if (server->conf->source) {
         source = resolvepassiveaddrinfo(server->conf->source, AF_UNSPEC, NULL, protodefs.socktype);
-        if(!source)
+        if (!source)
             debug(DBG_WARN, "tlsconnect: could not resolve source address to bind for server %s, using default", server->conf->name);
     }
 
@@ -126,24 +126,27 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
         gettimeofday(&now, NULL);
         if (timeout && (now.tv_sec - start.tv_sec) + wait > timeout) {
             debug(DBG_DBG, "tlsconnect: timeout");
-            if (source) freeaddrinfo(source);
+            if (source)
+                freeaddrinfo(source);
             return 0;
         }
-        if (wait) debug(DBG_INFO, "Next connection attempt to %s in %lds", server->conf->name, wait);
+        if (wait)
+            debug(DBG_INFO, "Next connection attempt to %s in %lds", server->conf->name, wait);
         sleep(wait);
         firsttry = 0;
 
         gettimeofday(&now, NULL);
         if (timeout && (now.tv_sec - start.tv_sec) > timeout) {
             debug(DBG_DBG, "tlsconnect: timeout");
-            if (source) freeaddrinfo(source);
+            if (source)
+                freeaddrinfo(source);
             return 0;
         }
 
         for (entry = list_first(server->conf->hostports); entry; entry = list_next(entry)) {
             hp = (struct hostportres *)entry->data;
             debug(DBG_INFO, "tlsconnect: trying to open TLS connection to server %s (%s port %s)", server->conf->name, hp->host, hp->port);
-            if ((server->sock = connecttcp(hp->addrinfo, source ? source : srcres, list_count(server->conf->hostports) > 1 ? 5 : 30)) < 0 ) {
+            if ((server->sock = connecttcp(hp->addrinfo, source ? source : srcres, list_count(server->conf->hostports) > 1 ? 5 : 30)) < 0) {
                 debug(DBG_ERR, "tlsconnect: TLS connection to %s (%s port %s) failed: TCP connect failed", server->conf->name, hp->host, hp->port);
                 goto concleanup;
             }
@@ -165,17 +168,22 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
                 goto concleanup;
             }
 
+            if (!SSL_set_ex_data(server->ssl, RSP_EX_DATA_CONFIG, server->conf)) {
+                debug(DBG_WARN, "tlsconnect: failed to set ex data");
+            }
+
             if (server->conf->sni) {
                 struct in6_addr tmp;
-                char *servername = server->conf->sniservername ? server->conf->sniservername : 
-                    server->conf->servername ? server->conf->servername :
-                    (inet_pton(AF_INET, hp->host, &tmp) || inet_pton(AF_INET6, hp->host, &tmp)) ? NULL : hp->host;
+                char *servername = server->conf->sniservername                                                   ? server->conf->sniservername
+                                   : server->conf->servername                                                    ? server->conf->servername
+                                   : (inet_pton(AF_INET, hp->host, &tmp) || inet_pton(AF_INET6, hp->host, &tmp)) ? NULL
+                                                                                                                 : hp->host;
                 if (servername && !tlssetsni(server->ssl, servername)) {
                     debug(DBG_ERR, "tlsconnect: set SNI %s failed", servername);
                     goto concleanup;
                 }
             }
-            
+
             SSL_set_fd(server->ssl, server->sock);
             if (sslconnecttimeout(server->ssl, 5) <= 0) {
                 while ((error = ERR_get_error()))
@@ -184,6 +192,18 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
                 goto concleanup;
             }
 
+            if (server->conf->pskid && server->conf->pskkey) {
+                if (SSL_session_reused(server->ssl)) {
+                    debug(DBG_WARN, "tlsconnect: TLS connection to %s (%s port %s), PSK identity %s with cipher %s up",
+                          server->conf->name, hp->host, hp->port, server->conf->pskid, SSL_CIPHER_get_name(SSL_get_current_cipher(server->ssl)));
+                    break;
+                } else {
+                    debug(DBG_ERR, "tlsconnect: TLS PSK set for %s (%s port %s) but not used in session, rejecting connection",
+                          server->conf->name, hp->host, hp->port);
+                    goto concleanup;
+                }
+            }
+
             cert = verifytlscert(server->ssl);
             if (!cert) {
                 debug(DBG_ERR, "tlsconnect: certificate verification failed for %s (%s port %s)", server->conf->name, hp->host, hp->port);
@@ -192,10 +212,10 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
 
             if (verifyconfcert(cert, server->conf, hp)) {
                 subj = getcertsubject(cert);
-                if(subj) {
+                if (subj) {
                     debug(DBG_WARN, "tlsconnect: TLS connection to %s (%s port %s), subject %s, %s with cipher %s up",
-                        server->conf->name, hp->host, hp->port, subj,
-                        SSL_get_version(server->ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(server->ssl)));
+                          server->conf->name, hp->host, hp->port, subj,
+                          SSL_get_version(server->ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(server->ssl)));
                     free(subj);
                 }
                 X509_free(cert);
@@ -205,12 +225,15 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
             }
             X509_free(cert);
 
-concleanup:
+        concleanup:
             /* ensure previous connection is properly closed */
             cleanup_connection(server);
         }
-        if (server->ssl) break;
+        if (server->ssl)
+            break;
     }
+    gettimeofday(&server->connecttime, NULL);
+    server->tlsnewkey = server->connecttime;
 
     origflags = fcntl(server->sock, F_GETFL, 0);
     if (origflags == -1) {
@@ -221,20 +244,26 @@ concleanup:
 
     pthread_mutex_lock(&server->lock);
     server->state = RSP_SERVER_STATE_CONNECTED;
-    gettimeofday(&server->connecttime, NULL);
     server->lostrqs = 0;
     pthread_mutex_unlock(&server->lock);
     pthread_mutex_lock(&server->newrq_mutex);
     server->conreset = reconnect;
     pthread_cond_signal(&server->newrq_cond);
     pthread_mutex_unlock(&server->newrq_mutex);
-    if (source) freeaddrinfo(source);
+    if (source)
+        freeaddrinfo(source);
     return 1;
 }
 
 int clientradputtls(struct server *server, unsigned char *rad, int radlen) {
     int cnt;
     struct clsrvconf *conf = server->conf;
+    struct timeval now;
+
+    if (radlen <= 0) {
+        debug(DBG_ERR, "clientradputtls: invalid buffer (length)");
+        return 0;
+    }
 
     pthread_mutex_lock(&server->lock);
     if (server->state != RSP_SERVER_STATE_CONNECTED) {
@@ -242,6 +271,17 @@ int clientradputtls(struct server *server, unsigned char *rad, int radlen) {
         return 0;
     }
 
+    gettimeofday(&now, NULL);
+#if OPENSSL_VERSION_NUMBER >= 0x10101000
+    if (now.tv_sec - server->tlsnewkey.tv_sec > RSP_TLS_REKEY_INTERVAL && SSL_version(server->ssl) >= TLS1_3_VERSION) {
+        debug(DBG_DBG, "clientradputtls: perform key update for long-running connection");
+        if (SSL_get_key_update_type(server->ssl) == SSL_KEY_UPDATE_NONE &&
+            !SSL_key_update(server->ssl, SSL_KEY_UPDATE_REQUESTED))
+            debug(DBG_WARN, "clientradputtls: request for key update failed for %s", conf->name);
+        server->tlsnewkey = now;
+    }
+#endif
+
     if ((cnt = sslwrite(server->ssl, rad, radlen, 0)) <= 0) {
         pthread_mutex_unlock(&server->lock);
         return 0;
@@ -255,33 +295,22 @@ int clientradputtls(struct server *server, unsigned char *rad, int radlen) {
 void *tlsclientrd(void *arg) {
     struct server *server = (struct server *)arg;
     unsigned char *buf = NULL;
-    struct timeval now;
     int len = 0;
 
     for (;;) {
-        len = radtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount+1), &server->lock, &buf);
-        if (!buf || !len) {
-            if (SSL_get_shutdown(server->ssl) || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF)) {
-                if (SSL_get_shutdown(server->ssl))
-                    debug (DBG_WARN, "tlsclientrd: connection to server %s lost", server->conf->name);
-                else if (server->lostrqs)
-                    debug (DBG_WARN, "tlsclientrd: server %s did not respond, closing connection.", server->conf->name);
-                if (server->dynamiclookuparg)
+        len = radtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount + 1), &server->lock, &buf);
+        if (buf && len > 0) {
+            if (!replyh(server, buf, len))
+                if (closeh(server))
                     break;
-                tlsconnect(server, 0, 1);
-            }
-            if (server->dynamiclookuparg) {
-                gettimeofday(&now, NULL);
-                if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
-                    debug(DBG_INFO, "tlsclientrd: idle timeout for %s", server->conf->name);
-                    break;
-                }
-            }
-            continue;
+            buf = NULL;
+        } else if (SSL_get_shutdown(server->ssl)) {
+            if (closeh(server))
+                break;
+        } else {
+            if (timeouth(server))
+                break;
         }
-
-        replyh(server, buf, len);
-        buf = NULL;
     }
     debug(DBG_INFO, "tlsclientrd: exiting for %s", server->conf->name);
     pthread_mutex_lock(&server->lock);
@@ -317,38 +346,54 @@ void *tlsservernew(void *arg) {
     s = *(int *)arg;
     free(arg);
     if (getpeername(s, (struct sockaddr *)&from, &fromlen)) {
-	debug(DBG_DBG, "tlsservernew: getpeername failed, exiting");
-	goto exit;
+        debug(DBG_DBG, "tlsservernew: getpeername failed, exiting");
+        goto exit;
     }
     debug(DBG_WARN, "tlsservernew: incoming TLS connection from %s", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
 
-    conf = find_clconf(handle, (struct sockaddr *)&from, &cur, &hp);
-    if (conf) {
-        pthread_mutex_lock(&conf->tlsconf->lock);
-        ctx = tlsgetctx(handle, conf->tlsconf);
-        if (!ctx) {
-            pthread_mutex_unlock(&conf->tlsconf->lock);
-            goto exit;
-        }
-
-        ssl = SSL_new(ctx);
-        pthread_mutex_unlock(&conf->tlsconf->lock);
-        if (!ssl)
-            goto exit;
-
-        SSL_set_fd(ssl, s);
-        if (sslaccepttimeout(ssl, 30) <= 0) {
-            while ((error = ERR_get_error()))
-                debug(DBG_ERR, "tlsservernew: SSL accept from %s failed: %s", conf->name, ERR_error_string(error, NULL));
-            debug(DBG_ERR, "tlsservernew: SSL_accept failed");
-            goto exit;
-        }
-        cert = verifytlscert(ssl);
-        if (!cert)
-            goto exit;
-        accepted_tls = conf->tlsconf;
+    if (!(conf = find_clconf(handle, (struct sockaddr *)&from, &cur, &hp))) {
+        debug(DBG_WARN, "tlsservernew: ignoring unknown TLS client %s", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
+        goto exit;
     }
 
+    pthread_mutex_lock(&conf->tlsconf->lock);
+    ctx = tlsgetctx(handle, conf->tlsconf);
+    if (!ctx) {
+        pthread_mutex_unlock(&conf->tlsconf->lock);
+        goto exit;
+    }
+
+    ssl = SSL_new(ctx);
+    pthread_mutex_unlock(&conf->tlsconf->lock);
+    if (!ssl)
+        goto exit;
+
+    if (!SSL_set_ex_data(ssl, RSP_EX_DATA_CONFIG_LIST, find_all_clconf(handle, (struct sockaddr *)&from, cur, &hp))) {
+        debug(DBG_WARN, "tlsservernew: failed to set ex data");
+    }
+
+    SSL_set_fd(ssl, s);
+    if (sslaccepttimeout(ssl, 30) <= 0) {
+        struct clsrvconf *selected = SSL_get_ex_data(ssl, RSP_EX_DATA_CONFIG);
+        conf = selected ? selected : conf;
+        while ((error = ERR_get_error()))
+            debug(DBG_ERR, "tlsservernew: SSL accept from %s (%s) failed: %s", conf->name, addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)), ERR_error_string(error, NULL));
+        debug(DBG_ERR, "tlsservernew: SSL_accept failed");
+        list_free(SSL_get_ex_data(ssl, RSP_EX_DATA_CONFIG_LIST));
+        goto exit;
+    }
+    list_free(SSL_get_ex_data(ssl, RSP_EX_DATA_CONFIG_LIST));
+
+    if (!(SSL_session_reused(ssl) || (cert = verifytlscert(ssl))))
+        goto exit;
+
+    {
+        struct clsrvconf *selected = SSL_get_ex_data(ssl, RSP_EX_DATA_CONFIG);
+        conf = selected ? selected : conf;
+    }
+
+    accepted_tls = conf->tlsconf;
+
     origflags = fcntl(s, F_GETFL, 0);
     if (origflags == -1) {
         debugerrno(errno, DBG_WARN, "Failed to get flags");
@@ -356,75 +401,82 @@ void *tlsservernew(void *arg) {
         debugerrno(errno, DBG_WARN, "Failed to set O_NONBLOCK");
     }
 
-    while (conf) {
-        if (accepted_tls == conf->tlsconf && verifyconfcert(cert, conf, NULL)) {
-            subj = getcertsubject(cert);
-            if(subj) {
-                debug(DBG_WARN, "tlsservernew: TLS connection from %s, client %s, subject %s, %s with cipher %s up",
-                    addr2string((struct sockaddr *)&from,tmp, sizeof(tmp)), conf->name, subj,
-                    SSL_get_version(ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(ssl)));
-                free(subj);
+    if (SSL_session_reused(ssl)) {
+        debug(DBG_WARN, "tlsservernew: TLS connection from %s, client %s, PSK identity %s wtih cipher %s up",
+              addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)), conf->name, conf->pskid,
+              SSL_CIPHER_get_name(SSL_get_current_cipher(ssl)));
+    } else {
+        while (conf) {
+            if (!conf->pskid && accepted_tls == conf->tlsconf && (verifyconfcert(cert, conf, NULL))) {
+                subj = getcertsubject(cert);
+                if (subj) {
+                    debug(DBG_WARN, "tlsservernew: TLS connection from %s, client %s, subject %s, %s with cipher %s up",
+                          addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)), conf->name, subj,
+                          SSL_get_version(ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(ssl)));
+                    free(subj);
+                }
+                X509_free(cert);
+                break;
             }
-            X509_free(cert);
-            client = addclient(conf, 1);
-            if (client) {
-                if (conf->keepalive)
-                    enable_keepalive(s);
-                client->ssl = ssl;
-                client->addr = addr_copy((struct sockaddr *)&from);
-                tlsserverrd(client);
-                removeclient(client);
-            } else
-                debug(DBG_WARN, "tlsservernew: failed to create new client instance");
-            goto exit;
+            conf = find_clconf(handle, (struct sockaddr *)&from, &cur, &hp);
         }
-        conf = find_clconf(handle, (struct sockaddr *)&from, &cur, &hp);
     }
-    debug(DBG_WARN, "tlsservernew: ignoring request, no matching TLS client for %s", 
-        addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
-    if (cert)
-	X509_free(cert);
+
+    if (!conf) {
+        debug(DBG_WARN, "tlsservernew: ignoring request, no matching TLS client for %s",
+              addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
+        goto exit;
+    }
+
+    client = addclient(conf, 1);
+    if (client) {
+        if (conf->keepalive)
+            enable_keepalive(s);
+        client->ssl = ssl;
+        client->addr = addr_copy((struct sockaddr *)&from);
+        tlsserverrd(client);
+        removeclient(client);
+    } else
+        debug(DBG_WARN, "tlsservernew: failed to create new client instance");
 
 exit:
+    if (cert)
+        X509_free(cert);
     if (ssl) {
-	SSL_shutdown(ssl);
-	SSL_free(ssl);
+        SSL_shutdown(ssl);
+        SSL_free(ssl);
     }
     shutdown(s, SHUT_RDWR);
     close(s);
     pthread_exit(NULL);
 }
 
-void *tlslistener(void *arg) {
-    pthread_t tlsserverth;
-    int s, *sp = (int *)arg, *s_arg = NULL;
-    struct sockaddr_storage from;
-    socklen_t fromlen = sizeof(from);
+void tlsaccept(int s) {
+    int *s_arg;
+    pthread_t tcpserverth;
 
-    listen(*sp, 128);
-
-    for (;;) {
-	s = accept(*sp, (struct sockaddr *)&from, &fromlen);
-	if (s < 0) {
-	    debug(DBG_WARN, "accept failed");
-	    continue;
-	}
-        s_arg = malloc(sizeof(s));
-        if (!s_arg)
-            debugx(1, DBG_ERR, "malloc failed");
-        *s_arg = s;
-	if (pthread_create(&tlsserverth, &pthread_attr, tlsservernew, (void *) s_arg)) {
-	    debug(DBG_ERR, "tlslistener: pthread_create failed");
-            free(s_arg);
-	    shutdown(s, SHUT_RDWR);
-	    close(s);
-	    continue;
-	}
-	pthread_detach(tlsserverth);
+    s_arg = malloc(sizeof(s));
+    if (!s_arg) {
+        debug(DBG_ERR, "tlsaccept: malloc failed");
+        return;
     }
-    free(sp);
+    *s_arg = s;
+    if (pthread_create(&tcpserverth, &pthread_attr, tlsservernew, (void *)s_arg)) {
+        debug(DBG_ERR, "tlsaccept: pthread_create failed");
+        free(s_arg);
+        shutdown(s, SHUT_RDWR);
+        close(s);
+        return;
+    }
+    pthread_detach(tcpserverth);
+}
+
+void *tlslistener(void *arg) {
+    accepttcp(*(int *)arg, tlsaccept);
+    free(arg);
     return NULL;
 }
+
 #else
 const struct protodefs *tlsinit(uint8_t h) {
     return NULL;
